@@ -1,9 +1,9 @@
-module ActionPack #:nodoc:
+module ActionPack
   module VERSION #:nodoc:
-    MAJOR = 2
-    MINOR = 2
-    TINY  = 2
-
+    MAJOR = 3
+    MINOR = 0
+    TINY  = 3
+    
     STRING = [MAJOR, MINOR, TINY].join('.')
   end
 end
